@@ -87,58 +87,6 @@ typedef enum
 
 G_DEFINE_TYPE (GspellInlineCheckerTextBuffer, _gspell_inline_checker_text_buffer, G_TYPE_OBJECT)
 
-/* FIXME properly. Workaround for bug in GtkTextView:
- * https://bugzilla.gnome.org/show_bug.cgi?id=763741
- */
-static void
-queue_draw_hack (GspellInlineCheckerTextBuffer *spell)
-{
-	GSList *l;
-
-	for (l = spell->views; l != NULL; l = l->next)
-	{
-		GtkWidget *view = GTK_WIDGET (l->data);
-		gtk_widget_queue_draw (view);
-	}
-}
-
-/* Remove the highlight_tag only if present. If gtk_text_buffer_remove_tag() is
- * called when the tag is not present, GtkTextView anyway queues a redraw, which
- * we want to avoid (it can lead to an infinite loop).
- */
-static void
-remove_highlight_tag_if_present (GspellInlineCheckerTextBuffer *spell,
-				 const GtkTextIter             *start,
-				 const GtkTextIter             *end)
-{
-	gboolean remove = FALSE;
-
-	if (gtk_text_iter_has_tag (start, spell->highlight_tag))
-	{
-		remove = TRUE;
-	}
-	else
-	{
-		GtkTextIter iter = *start;
-
-		if (gtk_text_iter_forward_to_tag_toggle (&iter, spell->highlight_tag) &&
-		    gtk_text_iter_compare (&iter, end) < 0)
-		{
-			remove = TRUE;
-		}
-	}
-
-	if (remove)
-	{
-		gtk_text_buffer_remove_tag (spell->buffer,
-					    spell->highlight_tag,
-					    start,
-					    end);
-
-		queue_draw_hack (spell);
-	}
-}
-
 static void
 check_word (GspellInlineCheckerTextBuffer *spell,
 	    const GtkTextIter             *start,
@@ -367,9 +315,8 @@ check_visible_region_in_view (GspellInlineCheckerTextBuffer *spell,
 					      &visible_start,
 					      &visible_end);
 
-	if (_gspell_region_is_empty (intersect))
+	if (intersect == NULL)
 	{
-		g_clear_object (&intersect);
 		return;
 	}
 
@@ -382,9 +329,10 @@ check_visible_region_in_view (GspellInlineCheckerTextBuffer *spell,
 						 &current_word_start,
 						 &current_word_end))
 		{
-			remove_highlight_tag_if_present (spell,
-							 &current_word_start,
-							 &current_word_end);
+			gtk_text_buffer_remove_tag (spell->buffer,
+						    spell->highlight_tag,
+						    &current_word_start,
+						    &current_word_end);
 
 			_gspell_region_subtract (intersect,
 						 &current_word_start,
@@ -396,15 +344,15 @@ check_visible_region_in_view (GspellInlineCheckerTextBuffer *spell,
 			_gspell_region_add (spell->scan_region,
 					    &current_word_start,
 					    &current_word_end);
-
-			if (_gspell_region_is_empty (intersect))
-			{
-				g_clear_object (&intersect);
-				return;
-			}
 		}
 	}
 
+	if (_gspell_region_is_empty (intersect))
+	{
+		g_clear_object (&intersect);
+		goto out;
+	}
+
 	_gspell_region_get_start_region_iter (intersect, &intersect_iter);
 
 	while (!_gspell_region_iter_is_end (&intersect_iter))
@@ -443,7 +391,14 @@ check_visible_region_in_view (GspellInlineCheckerTextBuffer *spell,
 		g_clear_object (&spell->scan_region);
 	}
 
-	queue_draw_hack (spell);
+out:
+	if (view != NULL)
+	{
+		/* FIXME properly. Workaround for bug in GtkTextView:
+		 * https://bugzilla.gnome.org/show_bug.cgi?id=612772
+		 */
+		gtk_widget_queue_draw (GTK_WIDGET (view));
+	}
 }
 
 static void
@@ -921,10 +876,9 @@ get_suggestion_menu (GspellInlineCheckerTextBuffer *spell,
 	return top_menu;
 }
 
-static void
-populate_popup_cb (GtkTextView                   *view,
-		   GtkMenu                       *menu,
-		   GspellInlineCheckerTextBuffer *spell)
+void
+_gspell_inline_checker_text_buffer_populate_popup (GspellInlineCheckerTextBuffer *spell,
+						   GtkMenu                       *menu)
 {
 	GtkWidget *menu_item;
 	GtkTextIter start;
@@ -941,11 +895,6 @@ populate_popup_cb (GtkTextView                   *view,
 		return;
 	}
 
-	/* Prepend separator */
-	menu_item = gtk_separator_menu_item_new ();
-	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), menu_item);
-	gtk_widget_show (menu_item);
-
 	/* Prepend suggestions */
 	menu_item = gtk_menu_item_new_with_mnemonic (_("_Spelling Suggestions..."));
 	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), menu_item);
@@ -1006,8 +955,6 @@ remove_tag_to_word (GspellInlineCheckerTextBuffer *spell,
 
 		iter = match_end;
 	}
-
-	queue_draw_hack (spell);
 }
 
 static void
@@ -1456,12 +1403,6 @@ _gspell_inline_checker_text_buffer_attach_view (GspellInlineCheckerTextBuffer *s
 				 spell,
 				 0);
 
-	g_signal_connect_object (view,
-				 "populate-popup",
-				 G_CALLBACK (populate_popup_cb),
-				 spell,
-				 0);
-
 	g_signal_connect_object (view,
 				 "draw",
 				 G_CALLBACK (draw_cb),
